<?php
/**
 * @package	RSTickets! Pro
 * @copyright	(c) 2010 - 2022 RSJoomla!
 * @link		https://www.rsjoomla.com
 * @license	GNU General Public License http://www.gnu.org/licenses/gpl-3.0.en.html
 */

// No direct access.
defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Uri\Uri;

/**
 * RSMembership! RSFinder! Plugin.
 */
class plgRsfinderRsticketspro extends CMSPlugin
{
    const EXTENSION = 'com_rsticketspro';
	
	 /**
     * @var bool
     */
    protected $autoloadLanguage = true;

    protected function canRun() {
        // Check if the component exists and is enabled - in backend
        return (
            file_exists(JPATH_ADMINISTRATOR . '/components/'.self::EXTENSION.'/helpers/rsticketspro.php') &&
            Factory::getApplication()->isClient('administrator') &&
            ComponentHelper::getComponent(self::EXTENSION, true)->enabled
        );
    }

    public function onRsfinderGetResults(&$tmp_results, $search, $limit, &$index)
    {
        if (!$this->canRun())
        {
            return;
        }

        // verify the index first, no need to continue if the limit is reached
        if ($index >= $limit)
        {
            return;
        }

        // Get the current User
        $user = Factory::getApplication()->getIdentity();
        $access  = $user->authorise('core.manage', self::EXTENSION) && $user->authorise('core.edit', self::EXTENSION);

        // if there is no access, skip the results
        if (!$access)
        {
            return;
        }

        $select_in = $this->params->get('searchList');

        // workaround for stored version
        if (is_string($select_in))
        {
            $select_in = explode(',', $select_in);
        }

        $output_results = array('dynamic' => array(), 'fixed' => array());

        // Get DBO.
        $db	= Factory::getContainer()->get('DatabaseDriver');

        if (in_array('tickets', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rst.id', 'value'),
                    $db->qn('rst.subject', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_tickets', 'rst'))
                ->where($db->qn('rst.subject') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_TICKET'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&view=ticket&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('departments', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstd.id', 'value'),
                    $db->qn('rstd.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_departments', 'rstd'))
                ->where($db->qn('rstd.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_DEPARTMENT'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=department.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('categories', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstkbc.id', 'value'),
                    $db->qn('rstkbc.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_kb_categories', 'rstkbc'))
                ->where($db->qn('rstkbc.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_CATEGORY'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=kbcategory.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('departments_custom_fields', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstcf.id', 'value'),
                    $db->qn('rstcf.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_custom_fields', 'rstcf'))
                ->where($db->qn('rstcf.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_DEPARTMENT_CUSTOM_FIELD'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=customfield.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('staff_groups', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstg.id', 'value'),
                    $db->qn('rstg.name', 'text')
                ]
            )
                ->from($db->qn('#__rsticketspro_groups', 'rstg'))
                ->where($db->qn('rstg.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_STAFF_GROUP'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=group.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('staff_members', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rsts.id', 'value'),
                    $db->qn('u.username', 'text'),
                    $db->qn('u.name', 'name')
                ]
            )
                ->from($db->qn('#__rsticketspro_staff', 'rsts'))
                ->join('left', $db->qn('#__users', 'u').' ON '.$db->qn('u.id').' = '.$db->qn('rsts.user_id'))
                ->where($db->qn('u.username') . " LIKE ".$db->q('%'.$search.'%')." OR ".$db->qn('u.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->name).' -> '.$this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_STAFF_MEMBER'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=staff.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('statuses', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rsts.id', 'value'),
                    $db->qn('rsts.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_statuses', 'rsts'))
                ->where($db->qn('rsts.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_STATUS'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=status.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('emails', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rste.id', 'value'),
                    $db->qn('rste.subject', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_emails', 'rste'))
                ->where($db->qn('rste.subject') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_EMAIL'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=email.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('articles', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstkbc.id', 'value'),
                    $db->qn('rstkbc.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_kb_content', 'rstkbc'))
                ->where($db->qn('rstkbc.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_ARTICLE'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=kbarticle.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && in_array('article_conversion_rules', $select_in))
        {
            $query = $db->getQuery(true);
            $query->select(
                [
                    $db->qn('rstkbr.id', 'value'),
                    $db->qn('rstkbr.name', 'text'),
                ]
            )
                ->from($db->qn('#__rsticketspro_kb_rules', 'rstkbr'))
                ->where($db->qn('rstkbr.name') . " LIKE ".$db->q('%'.$search.'%'))
                ->setLimit($limit);

            $db->setQuery($query);
            $results = $db->loadObjectList();

            if (!empty($results))
            {
                foreach($results as $result)
                {
                    // verify the index first
                    if ($index >= $limit)
                    {
                        break;
                    }

                    $output_result = array(
                        'title' => $this->escape($result->text).' '.Text::_('PLG_RSFINDER_RSTICKETSPRO_EDIT_ARTICLE_CONVERSION_RULE'),
                        'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&task=kbrule.edit&id='.$result->value,
                        'hits'  => 0
                    );

                    $output_results['dynamic'][] = $output_result;
                    $index ++;
                }
            }
        }

        if ($index < $limit && $this->params->get('configuration', 1))
        {
            $output_results['fixed'][] = array(
                'title' => Text::_('PLG_RSFINDER_RSTICKETSPRO_CONFIGURATION_PAGE'),
                'link'  => Uri::root().'administrator/index.php?option=com_rsticketspro&view=configuration',
                'hits'  => 0
            );
        }

        $tmp_results[] = $output_results;
    }

    protected function escape($string) {
        $string = (string) $string;
        return htmlentities($string, ENT_COMPAT, 'utf-8');
    }
        
}